#!/bin/env python
# -*- coding: utf-8 -*-

# Author: Grzegorz Szczudlik
# Date: 02.03.19 18:21

import requests

from nightcrawler import user_agent, logger
from nightcrawler.matchers import AnchorMatcher
from nightcrawler.utils import UniqueElasticList

matcher = AnchorMatcher()


class PageCrawler(object):
    log = logger.getLogger('request.PageCrawler')

    def __init__(self, url):
        self.__domain = url.netloc
        self.__queue = UniqueElasticList(url)
        self.__visited_urls = UniqueElasticList()

    def process(self):
        for url in self.__queue:
            self.__process(url)
        return self.__visited_urls[:]

    def __process(self, url):
        if url.netloc != self.__domain:
            return

        req = Request(url)
        for result in req.get_links():
            if result in self.__visited_urls:
                continue

            self.__queue.append(result)
        self.__visited_urls.append(url)


class Request(object):
    log = logger.getLogger('request.HTTPRequest')
    headers = {
        'user-agent': user_agent
    }

    def __init__(self, url):
        self.__url = url

    def get_links(self):
        self.log.debug("Making request to: %s" % self.__url.geturl())
        content = requests.get(self.__url.geturl(), headers=self.headers)
        return matcher.parse(content.text, self.__url) if content.status_code == 200 else []



#!/bin/env python
# -*- coding: utf-8 -*-

# Author: Grzegorz Szczudlik
# Date: 02.03.19 18:04

import logging as logger

version = "0.1.4"
user_agent = "NightCrawler/%s" % version

logger.basicConfig(level=logger.WARN,
                   format='%(name)-12s: %(levelname)-8s %(message)s',
                   datefmt='%m-%d %H:%M')



#!/bin/env python
# -*- coding: utf-8 -*-

# Author: Grzegorz Szczudlik
# Date: 03.03.19 14:38

from bs4 import BeautifulSoup
from nightcrawler import logger


class SitemapGenerator(object):
    log = logger.getLogger('sitemap.SitemapGenerator')
    namespace = "http://www.sitemaps.org/schemas/sitemap/0.9"

    def generate(self, result_set, fd):
        self.log.debug("Generating sitemap")
        xml = BeautifulSoup(features="lxml-xml")
        urlset = xml.new_tag("urlset", xmlns=self.namespace)

        for result in result_set:
            url = self.__generate_entry(xml, result)
            urlset.append(url)
            xml.append(urlset)

        self.log.debug("Writing map to disk")
        fd.write(xml.prettify())

        self.log.debug("Sitemap generated")

    def __generate_entry(self, soup, item):
        self.log.debug("Generating entry for %s" % item.geturl())
        url = soup.new_tag("url")

        loc = soup.new_tag("loc")
        loc.string = item.geturl()
        url.append(loc)

        return url



#!/bin/env python
# -*- coding: utf-8 -*-

# Author: Grzegorz Szczudlik
# Date: 02.03.19 22:33

import sys

if sys.version_info < (3, ):
    from urlparse import urlparse
else:
    from urllib.parse import urlparse


__all__ = ['ElasticList', 'UniqueElasticList', 'URL', 'InvalidURL']


# This class could implement more magic methods but for given task - it won't unless needed
class ElasticList(object):
    def __init__(self, *args):
        self._queue = []
        self._queue.extend(args)
        self._iter = None

    def push(self, item):
        self._queue.append(item)
    append = push

    def pop(self):
        if self._iter is not None:
            self._iter -= 1
        return self._queue.pop(0)

    def values(self):
        return self._queue[:]

    def __len__(self):
        return len(self._queue)

    def __contains__(self, item):
        return item in self._queue

    def __iter__(self):
        self._iter = 0
        return self

    def __getitem__(self, index):
        return self._queue[index]

    def __next__(self):
        if self._iter is None or len(self._queue)-1 < self._iter:
            self._iter = None
            raise StopIteration()

        value = self._queue[self._iter]
        self._iter += 1
        return value


class UniqueElasticList(ElasticList):
    def __init__(self, *args):
        super().__init__(*args)

    def push(self, item):
        if item not in self._queue:
            self._queue.append(item)
    append = push


class InsensitiveString(str):
    def __init__(self, value):
        super(str).__init__(value)
        self.__value = value

    def __eq__(self, other):
        return self.__value.lower == other.lower()


# Invalid URL Exception - to distinguish URL between validation and other errors
class InvalidURL(Exception):
    pass




#!/bin/env python
# -*- coding: utf-8 -*-

# Author: Grzegorz Szczudlik
# Date: 02.03.19 20:57

import sys

if sys.version_info < (3, ):
    from urlparse import urlparse, ParseResult
else:
    from urllib.parse import urlparse, ParseResult

from bs4 import BeautifulSoup

from nightcrawler import logger
from nightcrawler.utils import UniqueElasticList, InvalidURL


class AnchorMatcher(object):
    def __init__(self, list_class=UniqueElasticList):
        self.__list_cls = list_class

    def parse(self, content, url):
        results = self.__list_cls()

        log = logger.getLogger('matchers.AnchorMatcher')
        log.debug("Parsing content")

        parser = BeautifulSoup(content, "lxml")
        for item in parser.find_all("a"):
            if item is None:
                continue

            try:
                href = urlparse(item.get("href", ""))
            except InvalidURL:
                continue

            if href.scheme not in ('', 'http', 'https') or (href.netloc and href.netloc != url.netloc):
                continue

            href = ParseResult(
                scheme=href.scheme if href.scheme else url.scheme,
                netloc=href.netloc if href.netloc else url.netloc,
                path=href.path if href.path else "/",
                params=href.params,
                query=href.query,
                fragment=''
            )

            results.append(href)
            log.debug('Found url: {url}'.format(url=href.geturl()))

        return results



#!/bin/env python
# -*- coding: utf-8 -*-

# Author: Grzegorz Szczudlik
# Date: 02.03.2019 17:34

from setuptools import setup

import nightcrawler


def readme():
    with open('README.rst', 'r') as f:
        return f.read()


setup(
    name='NightCrawler',
    version=nightcrawler.version,
    description='Website crawling bot',
    long_description=readme(),
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.2',
    ],
    keywords='crawler spider website',
    url='https://github.com/szczad/NightCrawler',
    author='Grzeoorz Szczudlik',
    author_email='2914011+szczad@users.noreply.github.com',
    license='MIT',
    packages=['nightcrawler'],
    install_requires=[
        'beautifulsoup4',
        'requests',
        'lxml',
        'validators'
    ],
    tests_require=[
        'nose'
    ],
    test_suite='nose.collector',
    entry_points={
        'console_scripts': [
            'nightcrawler = nightcrawler.crawler:main'
        ]
    },
    zip_safe=True
)



#!/bin/env python
# -*- coding: utf-8 -*-

# Author: Grzegorz Szczudlik
# Date: 02.03.19 18:21

import argparse
import sys

if sys.version_info < (3, ):
    from urlparse import urlparse
else:
    from urllib.parse import urlparse


from validators import domain, url

from nightcrawler import logger
from nightcrawler.request import PageCrawler
from nightcrawler.sitemap import SitemapGenerator


def target_validator(target):
    if domain(target):
        # http:// should be starting point as secure pages will in fact redirect to https:// eventually
        return urlparse("http://%s/" % target)
    elif url(target):
        # Take into account only valid web protocols
        if target.startswith(("http://", "https://")):
            return urlparse(target)

    raise argparse.ArgumentTypeError("Invalid URL or domain provided: %s" % target)


def main():
    parser = argparse.ArgumentParser(description="Crawler that recursively search for links on sites")
    parser.add_argument("--output", "-o", default=sys.stdout,
                        help="Path to the file to store the output (default: STDOUT)")
    parser.add_argument("--debug", "-d", dest="level", action='store_const', default=logger.WARN, const=logger.DEBUG,
                        help="Be more verbose over what the script is doing")
    parser.add_argument("URL", metavar="URL|DOMAIN", nargs=1, type=target_validator, help="URL to start crawling from")

    args = parser.parse_args(sys.argv[1:])
    log = logger.getLogger('')
    log.setLevel(args.level)

    pc = PageCrawler(args.URL[0])
    mapping = pc.process()

    sg = SitemapGenerator()
    if isinstance(args.output, str):
        with open(args.output, 'w') as fd:
            sg.generate(mapping, fd)
    else:
        sg.generate(mapping, args.output)
